/*
 *	TXX EXTENSIONS -- external procedure interface for TXX
 *	Copyright (C) 1997-2002 Nils M Holm. All rights reserved.
 *	See the file LICENSE in the T3X documentation directory
 *	($T3XDIR)/docs for conditions of use.
 */

#define EXTERN extern

#ifdef MAIN
 #undef EXTERN
 #define EXTERN
#endif

#define UCHAR   unsigned char
#define USHORT  unsigned short
#define UINT    unsigned int
#define ULONG   unsigned long

#define MAXINFPROC	64
struct infproc {
	char	*name;
	int	(*proc)();
};

#ifdef __STDC__
 #ifndef COHERENT
  #define ANSIPROTO
 #endif
#else
 #ifdef __TURBOC__
  #define ANSIPROTO
 #endif
#endif

#ifdef __FreeBSD__
 #ifdef __i386__
  #define HOST32
  #define NL_IS_LF
 #else
  /* pre-3.0 FreeBSDs? */
  #ifdef i386
   #define HOST32
   #define NL_IS_LF
  #endif
  #ifdef __alpha__
   #define HOST64
   #define NL_IS_LF
  #endif
 #endif
#endif

#ifdef __NetBSD__
 #ifdef __alpha__
  #define HOST64
  #define NL_IS_LF
 #endif
 #ifdef mc68000
  #define HOST32
  #define NL_IS_LF
 #endif
#endif  

#ifdef COHERENT
 #ifdef _I386
  #define HOST32
  #define NL_IS_LF
  #define NEED_RENAME
  #define void char
  #include <sys/types.h>
 #endif
#endif

#ifdef __linux__
 #ifdef __i386__
  #define HOST32
  #define NL_IS_LF
 #endif
#endif  

#ifdef dos
 #define HOST16
 #define NL_IS_CRLF
 #include <dos.h>
 #ifdef __TURBOC__
  #define off_t	ULONG
  /* OOPS! open() and friends translate LF<-->CRLF */
  #define creat(X,Y)	_creat((X),0)
  #define open		_open
  #define close		_close
  #define read		_read
 #endif
#endif

#ifndef HOST16
 #ifndef HOST32
  #ifndef HOST64
   #error "Host machine word size undefined!"
  #endif
 #endif
#endif

#ifdef HOST16
 #define CELL	short
 #define UCELL	USHORT
 #define BPW	2
 #define WORDCAST	(short)
 #define XCELL	long
 #define XUCELL	ULONG
#endif

#ifdef HOST32
 #ifdef LIBRARY
  #define CELL	int
  #define UCELL	UINT
  #define BPW	4
 #else
  #define CELL	short
  #define UCELL	USHORT
  #define BPW	2
 #endif
 #define WORDCAST	(int)
 #define XCELL	int
 #define XUCELL	UINT
#endif

#ifdef HOST64
 #ifdef LIBRARY
  #define CELL	long
  #define UCELL	ULONG
  #define BPW	8
 #else
  #define CELL	short
  #define UCELL	USHORT
  #define BPW	2
 #endif
 #define WORDCAST	(long)
 #define XCELL	long
 #define XUCELL	ULONG
#endif

#ifndef LIBRARY
 #define ARG(C,N)	Stack[Sp+(C)-(N)]
 #define UARG(C,N)	Ustack[Sp+(C)-(N)]
 #define ARGP(C,N)	&Data[Ustack[Sp+(C)-(N)]]
#endif

#ifdef LIBRARY
 #define S0()		(__self)
 #define S1(x)		(__self,x)
 #define S2(x,y)	(__self,y,x)
 #define S3(x,y,z)	(__self,z,y,x)
 #define S4(x,y,z,a)	(__self,a,z,y,x)
#else
 #define S0()		()
 #define S1(x)		(x)
 #define S2(x,y)	(y,x)
 #define S3(x,y,z)	(z,y,x)
 #define S4(x,y,z,a)	(a,z,y,x)
#endif


EXTERN USHORT	*Ustack;
EXTERN short	*Stack;
EXTERN USHORT	Sp;
EXTERN UCHAR	*Data;
EXTERN short	Acc;
EXTERN char	**Argvec;
EXTERN int	Argcount;


#ifdef MAIN

void	libinit();

void	X_bpw(), X_close(), X_cvalist(), X_getarg(), X_getenv(),
	X_memcomp(), X_memcopy(), X_memfill(), X_memscan(),
	X_newline(), X_open(), X_read(), X_remove(),
	X_rename(),
	X_seek(), X_sys(), X_write();


void	(*Primitives[])() = {

	/*---TXX-PRIMITIVES-BEGIN---*/
	X_sys,
	X_bpw,
	X_memcopy,
	X_memcomp,
	X_memfill,
	X_memscan,
	X_getarg,
	X_getenv,
	X_open,
	X_read,
	X_write,
	X_newline,
	X_seek,
	X_close,
	X_rename,
	X_remove,
	X_cvalist,
	/*---TXX-PRIMITIVES-END---*/

	0
};

#endif	/* MAIN */
